@@ -1,11 +1,11 @@
-/*  $Id: http_header.pl,v 1.33 2007/04/10 20:40:48 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -34,41 +34,61 @@
 	    http_read_reply_header/2,	% +Stream, -Reply
 	    http_reply/2,		% +What, +Stream
 	    http_reply/3,		% +What, +Stream, +HdrExtra
+	    http_reply_header/3,	% +Stream, +What, +HdrExtra
 
 	    http_timestamp/2,		% +Time, -HTTP string
 
 	    http_post_data/3,		% +Stream, +Data, +HdrExtra
 
 	    http_read_header/2,		% +Fd, -Header
+	    http_parse_header/2,	% +Codes, -Header
 	    http_join_headers/3,	% +Default, +InHdr, -OutHdr
-	    http_update_encoding/3	% +HeaderIn, -Encoding, -HeaderOut
+	    http_update_encoding/3,	% +HeaderIn, -Encoding, -HeaderOut
+	    http_update_connection/4,	% +HeaderIn, +Request, -Connection, -HeaderOut
+	    http_update_transfer/4	% +HeaderIn, +Request, -Transfer, -HeaderOut
 	  ]).
 :- use_module(library(readutil)).
 :- use_module(library(debug)).
 :- use_module(library(lists)).
 :- use_module(library(url)).
 :- use_module(library(memfile)).
+:- use_module(library(settings)).
+:- use_module(library(error)).
 :- use_module(dcg_basics).
 :- use_module(html_write).
 :- use_module(mimetype).
 :- use_module(mimepack).
 
 
+% see http_update_transfer/4.
+
+:- setting(http:chunked_transfer, oneof([never,on_request,if_possible]),
+	   on_request, 'When to use Transfer-Encoding: Chunked').
+
+
 		 /*******************************
 		 *	    READ REQUEST	*
 		 *******************************/
 
-%%	http_read_request(+FdIn, -Request)
+%%	http_read_request(+FdIn:stream, -Request) is det.
 %
 %	Read an HTTP request-header from FdIn and return the broken-down
-%	request fields as +Name(+Value) pairs in a list.
+%	request fields as +Name(+Value) pairs  in   a  list.  Request is
+%	unified to =end_of_file= if FdIn is at the end of input.
 
-http_read_request(In, [input(In)|Request]) :-
+http_read_request(In, Request) :-
 	read_line_to_codes(In, Codes),
 	(   Codes == end_of_file
-	->  debug(http(header), 'end-of-file', [])
+	->  debug(http(header), 'end-of-file', []),
+	    Request = end_of_file
 	;   debug(http(header), 'First line: ~s~n', [Codes]),
-	    phrase(request(In, Request), Codes)
+	    Request =  [input(In)|Request1],
+	    phrase(request(In, Request1), Codes),
+	    (	Request1 = [unknown(Text)|_]
+	    ->	string_to_list(S, Text),
+		domain_error(http_request, S)
+	    ;	true
+	    )
 	).
 
 
@@ -149,7 +169,16 @@ http_reply(cgi_stream(In, Len), Out, HrdExtra) :- !,
 	phrase(reply_header(cgi_data(Size), Hdr2), Header),
 	format(Out, '~s', [Header]),
 	copy_stream_data(In, Out).
-http_reply(moved(To), Out, HrdExtra) :- !,
+http_reply(Status, Out, HdrExtra) :-
+	set_stream(Out, encoding(utf8)),
+	call_cleanup(http_status_reply(Status, Out, HdrExtra),
+		     set_stream(Out, encoding(octet))), !.
+
+%%	http_status_reply(+Status, +Out, +HdrExtra) is det.
+%
+%	Emit HTML non-200 status reports.
+
+http_status_reply(moved(To), Out, HrdExtra) :- !,
 	phrase(page([ title('301 Moved Permanently')
 		    ],
 		    [ h1('Moved Permanently'),
@@ -162,7 +191,7 @@ http_reply(moved(To), Out, HrdExtra) :- !,
 	phrase(reply_header(moved(To, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_reply(moved_temporary(To), Out, HrdExtra) :- !,
+http_status_reply(moved_temporary(To), Out, HrdExtra) :- !,
 	phrase(page([ title('302 Moved Temporary')
 		    ],
 		    [ h1('Moved Temporary'),
@@ -175,7 +204,7 @@ http_reply(moved_temporary(To), Out, HrdExtra) :- !,
 	phrase(reply_header(moved_temporary(To, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_reply(see_other(To),Out,HdrExtra) :- !,
+http_status_reply(see_other(To),Out,HdrExtra) :- !,
        phrase(page([ title('303 See Other')
                     ],
                     [ h1('See Other'),
@@ -188,7 +217,7 @@ http_reply(see_other(To),Out,HdrExtra) :- !,
         phrase(reply_header(see_other(To, HTML), HdrExtra), Header),
         format(Out, '~s', [Header]),
         print_html(Out, HTML).
-http_reply(not_found(URL), Out, HrdExtra) :- !,
+http_status_reply(not_found(URL), Out, HrdExtra) :- !,
 	phrase(page([ title('404 Not Found')
 		    ],
 		    [ h1('Not Found'),
@@ -201,7 +230,7 @@ http_reply(not_found(URL), Out, HrdExtra) :- !,
 	phrase(reply_header(status(not_found, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_reply(forbidden(URL), Out, HrdExtra) :- !,
+http_status_reply(forbidden(URL), Out, HrdExtra) :- !,
 	phrase(page([ title('403 Forbidden')
 		    ],
 		    [ h1('Forbidden'),
@@ -214,7 +243,7 @@ http_reply(forbidden(URL), Out, HrdExtra) :- !,
 	phrase(reply_header(status(forbidden, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_reply(authorise(Method, Realm), Out, HrdExtra) :- !,
+http_status_reply(authorise(Method, Realm), Out, HrdExtra) :- !,
 	phrase(page([ title('401 Authorization Required')
 		    ],
 		    [ h1('Authorization Required'),
@@ -231,7 +260,7 @@ http_reply(authorise(Method, Realm), Out, HrdExtra) :- !,
 	phrase(reply_header(authorise(Method, Realm, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_reply(not_modified, Out, HrdExtra) :- !,
+http_status_reply(not_modified, Out, HrdExtra) :- !,
 	phrase(page([ title('304 Not Modified')
 		    ],
 		    [ h1('Not Modified'),
@@ -242,7 +271,7 @@ http_reply(not_modified, Out, HrdExtra) :- !,
 	phrase(reply_header(status(not_modified, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
-http_reply(server_error(ErrorTerm), Out, HrdExtra) :-
+http_status_reply(server_error(ErrorTerm), Out, HrdExtra) :-
 	'$messages':translate_message(ErrorTerm, Lines, []),
 	phrase(page([ title('500 Internal server error')
 		    ],
@@ -254,6 +283,30 @@ http_reply(server_error(ErrorTerm), Out, HrdExtra) :-
 	phrase(reply_header(status(server_error, HTML), HrdExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
+http_status_reply(resource_error(ErrorTerm), Out, HrdExtra) :- !,
+	'$messages':translate_message(ErrorTerm, Lines, []),
+	phrase(page([ title('503 Service Unavailable')
+		    ],
+		    [ h1('Service Unavailable'),
+		      p(['The server is temporarily out of resources, please try again later']),
+		      p(\html_message_lines(Lines)),
+		      address(httpd)
+		    ]),
+	       HTML),
+	phrase(reply_header(status(service_unavailable, HTML), HrdExtra), Header),
+	format(Out, '~s', [Header]),
+	print_html(Out, HTML).
+http_status_reply(busy, Out, HrdExtra) :- !,
+	phrase(page([ title('503 Service Unavailable')
+		    ],
+		    [ h1('Service Unavailable'),
+		      p(['The server is temporarily out of resources, please try again later']),
+		      address(httpd)
+		    ]),
+	       HTML),
+	phrase(reply_header(status(service_unavailable, HTML), HrdExtra), Header),
+	format(Out, '~s', [Header]),
+	print_html(Out, HTML).
 
 
 html_message_lines([]) -->
@@ -306,13 +359,111 @@ http_update_encoding(Header0, utf8, [content_type(Type)|Header]) :-
 	;   B = Type0
 	),
 	atom_concat(B, '; charset=UTF-8', Type).
-http_update_encoding(Header, utf8, Header) :-
+http_update_encoding(Header, Encoding, Header) :-
 	memberchk(content_type(Type), Header),
-	(   sub_atom(Type, _, _, _, 'UTF-8')
-	;   sub_atom(Type, _, _, _, 'utf-8')
-	), !.
+	(   (   sub_atom(Type, _, _, _, 'UTF-8')
+	    ;   sub_atom(Type, _, _, _, 'utf-8')
+	    )
+	->  Encoding = utf8
+	;   mime_type_encoding(Type, Encoding)
+	).
 http_update_encoding(Header, octet, Header).
 
+%%	mime_type_encoding(+MimeType, -Encoding) is semidet.
+%
+%	Encoding is the (default) character encoding for MimeType.
+
+mime_type_encoding('application/json', utf8).
+mime_type_encoding('application/jsonrequest', utf8).
+
+
+%%	http_update_connection(+CGIHeader, +Request, -Connection, -Header)
+%
+%	Merge keep-alive information from  Request   and  CGIHeader into
+%	Header.
+
+http_update_connection(CgiHeader, Request, Connect, [connection(Connect)|Rest]) :-
+	select(connection(CgiConn), CgiHeader, Rest), !,
+	connection(Request, ReqConnection),
+	join_connection(ReqConnection, CgiConn, Connect).
+http_update_connection(CgiHeader, Request, Connect, [connection(Connect)|CgiHeader]) :-
+	connection(Request, Connect).
+
+join_connection(Keep1, Keep2, Connection) :-
+	(   downcase_atom(Keep1, 'keep-alive'),
+	    downcase_atom(Keep2, 'keep-alive')
+	->  Connection = 'Keep-Alive'
+	;   Connection = close
+	).
+
+
+%%	connection(+Header, -Connection)
+%	
+%	Extract the desired connection from a header.
+
+connection(Header, Close) :-
+	(   memberchk(connection(Connection), Header)
+	->  Close = Connection
+	;   memberchk(http_version(1-X), Header),
+	    X >= 1
+	->  Close = 'Keep-Alive'
+	;   Close = close
+	).
+
+
+%%	http_update_transfer(+Request, +CGIHeader, -Transfer, -Header)
+%
+%	Decide on the transfer encoding  from   the  Request and the CGI
+%	header.    The    behaviour    depends      on    the    setting
+%	http:chunked_transfer. If =never=, even   explitic  requests are
+%	ignored. If =on_request=, chunked encoding  is used if requested
+%	through  the  CGI  header  and  allowed    by   the  client.  If
+%	=if_possible=, chunked encoding is  used   whenever  the  client
+%	allows for it, which is  interpreted   as  the client supporting
+%	HTTP 1.1 or higher.
+%	
+%	Chunked encoding is more space efficient   and allows the client
+%	to start processing partial results. The drawback is that errors
+%	lead to incomplete pages instead of  a nicely formatted complete
+%	page.
+
+http_update_transfer(Request, CgiHeader, Transfer, Header) :-
+	setting(http:chunked_transfer, When),
+	http_update_transfer(When, Request, CgiHeader, Transfer, Header).
+
+http_update_transfer(never, _, CgiHeader, none, Header) :- !,
+	delete(CgiHeader, transfer_encoding(_), Header).
+http_update_transfer(_, Request, CgiHeader, Transfer, Header) :-
+	select(transfer_encoding(CgiTransfer), CgiHeader, Rest), !,
+	transfer(Request, ReqConnection),
+	join_transfer(ReqConnection, CgiTransfer, Transfer),
+	(   Transfer == none
+	->  Header = Rest
+	;   Header = [transfer_encoding(Transfer)|Rest]
+	).
+http_update_transfer(if_possible, Request, CgiHeader, Transfer, Header) :-
+	transfer(Request, Transfer),
+	Transfer \== none, !,
+	Header = [transfer_encoding(Transfer)|CgiHeader].
+http_update_transfer(_, _, CgiHeader, none, CgiHeader).
+
+join_transfer(chunked, chunked, chunked) :- !.
+join_transfer(_, _, none).
+
+
+%%	transfer(+Header, -Connection)
+%	
+%	Extract the desired connection from a header.
+
+transfer(Header, Transfer) :-
+	(   memberchk(transfer_encoding(Transfer0), Header)
+	->  Transfer = Transfer0
+	;   memberchk(http_version(1-X), Header),
+	    X >= 1
+	->  Transfer = chunked
+	;   Transfer = none
+	).
+
 
 %%	content_length_in_encoding(+Encoding, +In, -Bytes)
 %	
@@ -382,7 +533,11 @@ content_length_in_encoding(Enc, Stream, Bytes) :-
 %	  multipart/mixed and packed using mime_pack/3. See mime_pack/3
 %	  for details on the argument format.
 
+:- multifile
+	http_client:post_data_hook/3.
 
+http_post_data(Data, Out, HdrExtra) :-
+	http_client:post_data_hook(Data, Out, HdrExtra), !.
 http_post_data(html(HTML), Out, HdrExtra) :-
 	phrase(post_header(html(HTML), HdrExtra), Header),
 	format(Out, '~s', [Header]),
@@ -412,7 +567,7 @@ http_post_data(cgi_stream(In), Out, HdrExtra) :- !,
 	http_update_encoding(Header0, Encoding, Header),
 	content_length_in_encoding(Encoding, In, Size),
 	http_join_headers(HdrExtra, Header, Hdr2),
-	phrase(reply_header(cgi_data(Size), Hdr2), HeaderText),
+	phrase(post_header(cgi_data(Size), Hdr2), HeaderText),
 	format(Out, '~s', [HeaderText]),
 	set_stream(Out, encoding(Encoding)),
 	call_cleanup(copy_stream_data(In, Out),
@@ -462,7 +617,10 @@ http_post_data(List, Out, HdrExtra) :-		% multipart-mixed
 	close(In),
 	free_memory_file(MemFile).
 
-%	post_header//2: DCG for generating the POST request
+%%	post_header(+Data, +HeaderExtra)//
+%
+%	Generate the POST header, emitting HeaderExtra, followed by the
+%	HTTP Content-length and Content-type fields.
 
 post_header(html(Tokens), HdrExtra) -->
 	header_fields(HdrExtra),
@@ -489,6 +647,16 @@ post_header(codes(Type, Codes), HdrExtra) -->
 		 *       OUTPUT HEADER DCG	*
 		 *******************************/
 
+%%	http_reply_header(+Out:stream, +What, +HdrExtra) is det.
+%
+%	Create a reply header  using  reply_header//2   and  send  it to
+%	Stream.
+
+http_reply_header(Out, What, HdrExtra) :-
+	phrase(reply_header(What, HdrExtra), String), !,
+	format(Out, '~s', [String]).
+	
+
 reply_header(string(String), HdrExtra) -->
 	reply_header(string(text/plain, String), HdrExtra).
 reply_header(string(Type, String), HdrExtra) -->
@@ -526,13 +694,22 @@ reply_header(cgi_data(Size), HdrExtra) -->
 	header_fields(HdrExtra),
 	content_length(Size),
 	"\r\n".
+reply_header(chunked_data, HdrExtra) -->
+	vstatus(ok),
+	date(now),
+	header_fields(HdrExtra),
+	(   {memberchk(transfer_encoding(_), HdrExtra)}
+	->  ""
+	;   transfer_encoding(chunked)
+	),
+	"\r\n".
 reply_header(moved(To, Tokens), HdrExtra) -->
 	vstatus(moved),
 	date(now),
 	header_field('Location', To),
 	header_fields(HdrExtra),
 	content_length(html(Tokens)),
-	content_type(text/html),
+	content_type(text/html, utf8),
 	"\r\n".
 reply_header(moved_temporary(To, Tokens), HdrExtra) -->
 	vstatus(moved_temporary),
@@ -540,22 +717,22 @@ reply_header(moved_temporary(To, Tokens), HdrExtra) -->
 	header_field('Location', To),
 	header_fields(HdrExtra),
 	content_length(html(Tokens)),
-	content_type(text/html),
+	content_type(text/html, utf8),
 	"\r\n".
 reply_header(see_other(To,Tokens),HdrExtra) -->
-      vstatus(see_other),
-      date(now),
-      header_field('Location',To),
-      header_fields(HdrExtra),
-      content_length(html(Tokens)),
-      content_type(text/html),
-      "\r\n".
+	vstatus(see_other),
+	date(now),
+	header_field('Location',To),
+	header_fields(HdrExtra),
+	content_length(html(Tokens)),
+	content_type(text/html, utf8),
+	"\r\n".
 reply_header(status(Status, Tokens), HdrExtra) -->
 	vstatus(Status),
 	date(now),
 	header_fields(HdrExtra),
 	content_length(html(Tokens)),
-	content_type(text/html),
+	content_type(text/html, utf8),
 	"\r\n".
 reply_header(authorise(Method, Realm, Tokens), HdrExtra) -->
 	vstatus(authorise),
@@ -563,7 +740,7 @@ reply_header(authorise(Method, Realm, Tokens), HdrExtra) -->
 	authenticate(Method, Realm),
 	header_fields(HdrExtra),
 	content_length(html(Tokens)),
-	content_type(text/html),
+	content_type(text/html, utf8),
 	"\r\n".
 
 vstatus(Status) -->
@@ -573,16 +750,21 @@ vstatus(Status) -->
 	status_comment(Status),
 	"\r\n".
 
-status_number(continue)	       --> "100".
-status_number(ok)	       --> "200".
-status_number(moved)	       --> "301".
-status_number(moved_temporary) --> "302".
-status_number(see_other)       --> "303".
-status_number(not_modified)    --> "304". 
-status_number(not_found)       --> "404".
-status_number(forbidden)       --> "403".
-status_number(authorise)       --> "401".
-status_number(server_error)    --> "500".
+%%	status_number(-Code)//
+%
+%	Parse the HTTP status numbers and return them as a code (atom).
+
+status_number(continue)		   --> "100".
+status_number(ok)		   --> "200".
+status_number(moved)		   --> "301".
+status_number(moved_temporary)	   --> "302".
+status_number(see_other)	   --> "303".
+status_number(not_modified)	   --> "304". 
+status_number(not_found)	   --> "404".
+status_number(forbidden)	   --> "403".
+status_number(authorise)	   --> "401".
+status_number(server_error)	   --> "500".
+status_number(service_unavailable) --> "503".
 
 status_comment(continue) -->
 	"Continue".
@@ -593,7 +775,7 @@ status_comment(moved) -->
 status_comment(moved_temporary) -->
 	"Moved Temporary".
 status_comment(see_other) -->
-      "See Other".
+	"See Other".
 status_comment(not_modified) --> 
 	"Not Modified".
 status_comment(not_found) -->
@@ -604,6 +786,8 @@ status_comment(authorise) -->
 	"Authorization Required".
 status_comment(server_error) -->
 	"Internal Server Error".
+status_comment(service_unavailable) -->
+	"Service Unavailable".
 
 authenticate(Method, '') --> !,
 	"WWW-Authenticate: ",
@@ -650,33 +834,63 @@ content_length(Len) -->
 	"Content-Length: ", string(LenChars),
 	"\r\n".
 
-content_type(Main/Sub) --> !,
+transfer_encoding(Encoding) -->
+	"Transfer-Encoding: ", atom(Encoding), "\r\n".
+
+content_type(Type) -->
+	content_type(Type, _).
+
+content_type(Type, Charset) -->
+	ctype(Type),
+	charset(Charset),
+	"\r\n".
+
+ctype(Main/Sub) --> !,
 	"Content-Type: ",
 	atom(Main),
 	"/",
-	atom(Sub),
-	"\r\n".
-content_type(Type) --> !,
+	atom(Sub).
+ctype(Type) --> !,
 	"Content-Type: ",
-	atom(Type),
-	"\r\n".
+	atom(Type).
+
+charset(Var) --> 
+	{ var(Var) }, !.
+charset(utf8) --> !,
+	"; charset=UTF-8".
+charset(CharSet) -->
+	"; charset=",
+	atom(CharSet).
 
 header_field(Name, Value) -->
-	{ var(Name)
-	}, !,
+	{ var(Name) }, !,
 	field_name(Name),
 	":",
 	whites,
-	string(ValueChars),
+	read_field_value(ValueChars),
 	blanks_to_nl, !,
-	{ field_to_prolog(Name, ValueChars, Value)
-	}.
+	{ field_to_prolog(Name, ValueChars, Value) }.
 header_field(Name, Value) -->
 	field_name(Name),
 	": ",
 	field_value(Value),
 	"\r\n".
 
+%%	read_field_value(-Codes)//
+%
+%	Read a field eagerly upto the next whitespace
+
+read_field_value([H|T]) -->
+	[H],
+	{ \+ code_type(H, space) }, !,
+	read_field_value(T).
+read_field_value([]) -->
+	"".
+read_field_value([H|T]) -->
+	[H],
+	read_field_value(T).
+
+
 field_to_prolog(content_length, ValueChars, ContentLength) :- !,
 	number_codes(ContentLength, ValueChars).
 field_to_prolog(cookie, ValueChars, Cookies) :- !,
@@ -721,6 +935,7 @@ header_fields([H|T]) -->
 	{ H =.. [Name, Value]
 	},
 	field_name(Name),
+	whites,
 	": ",
 	field_value(Value),
 	"\r\n",
@@ -731,34 +946,38 @@ header_fields([H|T]) -->
 %	Convert between prolog_name and HttpName
 
 field_name(Name) -->
-	{ var(Name)
-	}, !,
-	rd_field_chars(0':, Chars),
-	{ atom_codes(Name, Chars)
-	}.
+	{ var(Name) }, !,
+	rd_field_chars(Chars),
+	{ atom_codes(Name, Chars) }.
 field_name(mime_version) --> !,
 	"MIME-Version".
 field_name(Name) -->
-	{ atom_codes(Name, Chars)
-	},
+	{ atom_codes(Name, Chars) },
 	wr_field_chars(Chars).
 
-rd_field_chars(End, [C0|T]) -->
+rd_field_chars([C0|T]) -->
 	[C],
-	{ C \== End, !,
-	  (   C == 0'-
-	  ->  C0 = 0'_
-	  ;   code_type(C, to_upper(C0))
-	  )
-	},
-	rd_field_chars(End, T).
-rd_field_chars(_, []) -->
+	{ rd_field_char(C, C0) }, !,
+	rd_field_chars(T).
+rd_field_chars([]) -->
 	[].
 
+term_expansion(rd_field_char(_,_), Clauses) :-
+	Clauses = [ rd_field_char(0'-, 0'_)
+		  | Cls
+		  ],
+	findall(rd_field_char(In, Out),
+		(   between(1, 127, In),
+		    code_type(In, csym),
+		    code_type(Out, to_lower(In))),
+		Cls).
+
+rd_field_char(_, _).
+
+
 wr_field_chars([C|T]) -->
 	[C2], !,
-	{ to_lower(C2, C)
-	},
+	{ to_lower(C2, C) },
 	wr_field_chars2(T).
 wr_field_chars([]) -->
 	[].
@@ -791,13 +1010,13 @@ rfc_date(Time, String, Tail) :-
 		    '%a, %d %b %Y %H:%M:%S GMT',
 		    Date, posix).
 	
-%%	http_timestamp(+Time, -Atom)
+%%	http_timestamp(+Time:timestamp, -Text:atom) is det.
 %	
 %	Generate a description of a Time in HTTP format (RFC1123)
 
 http_timestamp(Time, Atom) :-
 	stamp_date_time(Time, Date, 'UTC'),
-	format_time(codes(Atom),
+	format_time(atom(Atom),
 		    '%a, %d %b %Y %H:%M:%S GMT',
 		    Date, posix).
 
@@ -875,11 +1094,9 @@ cookie(Name, Value) -->
 	cookie_value(Value).
 
 cookie_name(Name) -->
-	{ var(Name)
-	}, !,
-	rd_field_chars(0'=, Chars),
-	{ atom_codes(Name, Chars)
-	}.
+	{ var(Name) }, !,
+	rd_field_chars(Chars),
+	{ atom_codes(Name, Chars) }.
 
 cookie_value(Value) -->
 	chars_to_semicolon_or_blank(Chars),
@@ -917,7 +1134,7 @@ cookie_options([]) -->
 cookie_option(secure=true) -->
 	"secure", !.
 cookie_option(Name=Value) -->
-	rd_field_chars(0'=, NameChars),
+	rd_field_chars(NameChars), whites,
 	"=", blanks,
 	chars_to_semicolon(ValueChars),
 	{ atom_codes(Name, NameChars),
@@ -938,9 +1155,17 @@ chars_to_semicolon([]) -->
 		 *	     REPLY DCG		*
 		 *******************************/
 
-%	Typical reply:
+%%	reply(+In, -Reply:list)// is semidet.
 %
-%	HTTP/1.1 200 OK
+%	Process the first line of an HTTP   reply.  After that, read the
+%	remainder  of  the  header  and    parse  it.  After  successful
+%	completion, Reply contains the following fields, followed by the
+%	fields produced by http_read_header/2.
+%	
+%	    * http_version(Major-Minor)
+%	    * status(StatusCode, Comment)
+%	    
+%	StatusCode is one of the values provided by status_number//1.
 
 reply(Fd, [http_version(HttpVersion), status(Status, Comment)|Header]) -->
 	http_version(HttpVersion),
@@ -949,10 +1174,12 @@ reply(Fd, [http_version(HttpVersion), status(Status, Comment)|Header]) -->
 	->  []
 	;   integer(Status)
 	),
-	string(Comment),
-	blanks_to_nl,
 	blanks,
-	{ http_read_header(Fd, Header)
+	string(CommentCodes),
+	blanks_to_nl, !,
+	blanks,
+	{ atom_codes(Comment, CommentCodes),
+	  http_read_header(Fd, Header)
 	}.
 
 
@@ -960,7 +1187,7 @@ reply(Fd, [http_version(HttpVersion), status(Status, Comment)|Header]) -->
 		 *	      READ HEADER	*
 		 *******************************/
 
-%%	http_read_header(+Fd, -Header)
+%%	http_read_header(+Fd, -Header) is det.
 %
 %	Read Name: Value lines from FD until an empty line is encountered.
 %	Field-name are converted to Prolog conventions (all lower, _ instead
@@ -968,7 +1195,7 @@ reply(Fd, [http_version(HttpVersion), status(Status, Comment)|Header]) -->
 
 http_read_header(Fd, Header) :-
 	read_header_data(Fd, Text),
-	parse_header(Text, Header).
+	http_parse_header(Text, Header).
 
 read_header_data(Fd, Header) :-
 	read_line_to_codes(Fd, Header, Tail),
@@ -982,7 +1209,7 @@ read_header_data(_, Fd, Tail) :-
 	read_line_to_codes(Fd, Tail, NewTail),
 	read_header_data(Tail, Fd, NewTail).
 
-parse_header(Text, Header) :-
+http_parse_header(Text, Header) :-
 	phrase(header(Header), Text),
 	debug(http(header), 'Fields: ~w~n', [Header]).
 
