@@ -178,7 +178,6 @@ static const PL_extension foreigns[] = {
   FRG("$qlf_end_part",		0, pl_qlf_end_part,		0),
   FRG("$qlf_open",		1, pl_qlf_open,			0),
   FRG("$qlf_close",		0, pl_qlf_close,		0),
-  FRG("$qlf_load",		2, pl_qlf_load,		     META),
   FRG("$qlf_assert_clause",	2, pl_qlf_assert_clause,	0),
 
   FRG("abolish",    		1, pl_abolish1,		     META|ISO),
@@ -189,15 +188,6 @@ static const PL_extension foreigns[] = {
   FRG("nth_clause", 		3, pl_nth_clause,       NDET|META|CREF),
   FRG("retract",    		1, pl_retract,          NDET|META|CREF|ISO),
   FRG("retractall",		1, pl_retractall,	 META|ISO),
-#if O_DEBUGGER
-  FRG("$xr_member",		2, pl_xr_member,        NDET|META),
-  FRG("$wam_list",		1, pl_wam_list,			0),
-  FRG("$fetch_vm",		4, pl_fetch_vm,			0),
-  FRG("$clause_term_position",	3, pl_clause_term_position,	0),
-  FRG("$break_pc",		3, pl_break_pc,		     NDET),
-  FRG("$break_at",		3, pl_break_at,			0),
-  FRG("$current_break",		2, pl_current_break,	     NDET),
-#endif  
 #ifdef O_MAINTENANCE
   FRG("$list_generations",	1, pl_list_generations,	     META),
   FRG("$check_procedure",	1, pl_check_procedure,	     META),
@@ -362,7 +352,6 @@ static const PL_extension foreigns[] = {
   FRG("thread_create",		3, pl_thread_create,	 META|ISO),
   FRG("thread_join",		2, pl_thread_join,	      ISO),
   FRG("thread_exit",		1, pl_thread_exit,		0),
-  FRG("thread_kill",		2, pl_thread_kill,              0),
   FRG("thread_signal",		2, pl_thread_signal,	 META|ISO),
   FRG("thread_at_exit",		1, pl_thread_at_exit,	     META),
   FRG("mutex_destroy",		1, pl_mutex_destroy,	      ISO),
@@ -523,7 +512,6 @@ DECL_PLIST(list);
 DECL_PLIST(module);
 DECL_PLIST(prims);
 DECL_PLIST(read);
-DECL_PLIST(wam);
 DECL_PLIST(thread);
 DECL_PLIST(profile);
 DECL_PLIST(wic);
@@ -553,7 +541,6 @@ initBuildIns(void)
   REG_PLIST(module);
   REG_PLIST(prims);
   REG_PLIST(read);
-  REG_PLIST(wam);
   REG_PLIST(thread);
   REG_PLIST(profile);
   REG_PLIST(wic);
