@@ -1,4 +1,4 @@
-/*  $Id: pl-fli.c,v 1.185 2007/06/19 09:05:31 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -38,9 +38,8 @@
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SWI-Prolog  new-style  foreign-language  interface.   This  new  foreign
 interface is a mix of the old  interface using the ideas on term-handles
-from  Quintus  Prolog.  Term-handles  are    integers  (uintptr_t),
-describing the offset of the term-location relative   to the base of the
-local stack.
+from Quintus Prolog. Term-handles are   integers (uintptr_t), describing
+the offset of the term-location relative to the base of the local stack.
 
 If a C-function has to  store  intermediate   results,  it  can do so by
 creating a new term-reference using   PL_new_term_ref().  This functions
@@ -110,6 +109,9 @@ linkVal__LD(Word p ARG_LD)
     w = *p;
   }
 
+#ifdef LIFE_GC
+  SECURE(assert(w != ATOM_garbage_collected));
+#endif
   return w;
 }
 
@@ -1124,7 +1126,8 @@ PL_get_list_chars(term_t l, char **s, unsigned flags)
 
 int
 PL_get_wchars(term_t l, size_t *length, pl_wchar_t **s, unsigned flags)
-{ PL_chars_t text;
+{ GET_LD
+  PL_chars_t text;
 
   if ( !PL_get_text(l, &text, flags) )
     return FALSE;
@@ -1142,7 +1145,8 @@ PL_get_wchars(term_t l, size_t *length, pl_wchar_t **s, unsigned flags)
 
 int
 PL_get_nchars(term_t l, size_t *length, char **s, unsigned flags)
-{ PL_chars_t text;
+{ GET_LD
+  PL_chars_t text;
 
   if ( !PL_get_text(l, &text, flags) )
     return FALSE;
@@ -1325,6 +1329,17 @@ PL_get_int64(term_t t, int64_t *i)
 #define PL_get_int64(t, i) PL_get_int64__LD(t, i PASS_LD)
 
 
+int
+PL_get_intptr(term_t t, intptr_t *i)
+{ GET_LD
+#if SIZEOF_LONG != SIZEOF_VOIDP && SIZEOF_VOIDP == 8
+   return PL_get_int64(t, i);
+#else
+   return PL_get_long(t, (long*)i);
+#endif
+}
+
+
 int
 PL_is_inf(term_t t)
 { GET_LD
@@ -1403,13 +1418,17 @@ PL_get_name_arity(term_t t, atom_t *name, int *arity)
   if ( isTerm(w) )
   { FunctorDef fd = valueFunctor(functorTerm(w));
 
-    *name =  fd->name;
-    *arity = fd->arity;
+    if ( name )
+      *name =  fd->name;
+    if ( arity )
+      *arity = fd->arity;
     succeed;
   }
   if ( isTextAtom(w) )
-  { *name = (atom_t)w;
-    *arity = 0;
+  { if ( name )
+      *name = (atom_t)w;
+    if ( arity )
+      *arity = 0;
     succeed;
   }
 
@@ -2016,7 +2035,13 @@ PL_put_int64(term_t t, int64_t i)
 
 void
 _PL_put_number__LD(term_t t, Number n ARG_LD)
-{ setHandle(t, put_number(n));
+{ word w;
+
+  if ( (w=put_number(n)) )
+  { setHandle(t, w);
+  } else
+  { assert(0);
+  }
 }
 
 
@@ -2248,7 +2273,7 @@ codeToAtom(int chrcode)
     }
 
     if ( !(a=pv[entry]) )
-    { a = uncachedCodeToAtom(chrcode);
+    { a = pv[entry] = uncachedCodeToAtom(chrcode);
     }
   } else
   { a = uncachedCodeToAtom(chrcode);
@@ -2329,11 +2354,13 @@ flags: bitwise or of type and representation
 
 	Types:		PL_ATOM, PL_STRING, PL_CODE_LIST, PL_CHAR_LIST
 	Representation: REP_ISO_LATIN_1, REP_UTF8, REP_MB
+	Extra:		PL_DIFF_LIST
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
 PL_unify_chars(term_t t, int flags, size_t len, const char *s)
 { PL_chars_t text;
+  term_t tail;
   int rc;
 
   if ( len == (size_t)-1 )
@@ -2348,7 +2375,14 @@ PL_unify_chars(term_t t, int flags, size_t len, const char *s)
 
   flags &= ~(REP_UTF8|REP_MB|REP_ISO_LATIN_1);
 
-  rc = PL_unify_text(t, 0, &text, flags);
+  if ( (flags & PL_DIFF_LIST) )
+  { tail = t+1;
+    flags &= (~PL_DIFF_LIST);
+  } else
+  { tail = 0;
+  }
+
+  rc = PL_unify_text(t, tail, &text, flags);
   PL_free_text(&text);
 
   return rc;
@@ -2378,6 +2412,39 @@ PL_unify_int64(term_t t, int64_t i)
 }
 
 
+int
+PL_unify_int64_ex__LD(term_t t, int64_t i ARG_LD)
+{ word w = consInt(i);
+
+  if ( valInt(w) == i )
+  { Word p = valHandleP(t);
+
+    deRef(p);
+    if ( *p == w )
+      succeed;
+    if ( canBind(*p) )
+    { bindConst(p, w);
+      succeed;
+    }
+    if ( isInteger(*p) )
+      fail;
+  } else
+  { Word p = valHandleP(t);
+
+    deRef(p);
+    if ( canBind(*p) )
+    { w = globalLong(i PASS_LD);
+      bindConst(p, w);
+      succeed;
+    }
+    if ( isInteger(*p) )
+      return valInt(*p) == i ? TRUE : FALSE;
+  }
+  
+  return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, t);
+}
+
+
 int
 PL_unify_pointer__LD(term_t t, void *ptr ARG_LD)
 { word w = makeNum(pointerToLong(ptr));
@@ -3106,23 +3173,6 @@ PL_foreign_control(control_t h)
 }
 
 
-static QueryFrame
-find_query(LocalFrame fr)
-{ if ( fr )
-  { QueryFrame qf;
-
-    while(fr->parent)
-      fr = fr->parent;
-    
-    qf = (QueryFrame)addPointer(fr, -(intptr_t)offsetof(struct queryFrame, frame));
-
-    return qf;
-  }
-
-  return NULL;
-}
-
-
 int
 PL_raise_exception(term_t exception)
 { GET_LD
@@ -3366,8 +3416,9 @@ PL_open_resource(Module m,
 int
 PL_raise(int sig)
 { GET_LD
+
   if ( sig > 0 && sig <= MAXSIGNAL && LD )
-  { LD->pending_signals |= (1L << (sig-1));
+  { LD->pending_signals |= ((int64_t)1 << (sig-1));
     return TRUE;
   }
 
@@ -3375,6 +3426,25 @@ PL_raise(int sig)
 }
 
 
+int
+PL_pending__LD(int sig ARG_LD)
+{ if ( sig > 0 && sig <= MAXSIGNAL && LD )
+    return (LD->pending_signals & ((int64_t)1 << (sig-1))) ? TRUE : FALSE;
+
+  return -1;
+}
+
+
+int
+PL_clearsig__LD(int sig ARG_LD)
+{ if ( sig > 0 && sig <= MAXSIGNAL && LD )
+  { LD->pending_signals &= ~((int64_t)1 << (sig-1));
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 		/********************************
 		*         RESET (ABORTS)	*
 		********************************/
@@ -3723,7 +3793,7 @@ PL_fatal_error(const char *fm, ...)
 
 int
 PL_action(int action, ...)
-{ int rval;
+{ int rval = TRUE;
   va_list args;
 
   va_start(args, action);
@@ -3734,7 +3804,6 @@ PL_action(int action, ...)
       break;
     case PL_ACTION_DEBUG:
       debugmode(DBG_ALL, NULL);
-      rval = TRUE;
       break;
     case PL_ACTION_BACKTRACE:
 #ifdef O_DEBUGGER
@@ -3746,17 +3815,18 @@ PL_action(int action, ...)
       { Sfprintf(Serror,
 		 "\n[Cannot print stack while in %ld-th garbage collection]\n",
 		 gc_status.collections);
-	fail;
+	rval = FALSE;
+	break;
       }
       if ( GD->bootsession || !GD->initialised )
       { Sfprintf(Serror,
 		 "\n[Cannot print stack while initialising]\n");
-	fail;
+	rval = FALSE;
+	break;
       }
       om = systemMode(TRUE);		/* Also show hidden frames */
       backTrace(environment_frame, a);
       systemMode(om);
-      rval = TRUE;
     }
 #else
       warning("No Prolog backtrace in runtime version");
@@ -3779,7 +3849,6 @@ PL_action(int action, ...)
     case PL_ACTION_GUIAPP:
     { int guiapp = va_arg(args, int);
       GD->os.gui_app = guiapp;
-      rval = TRUE;
       break;
     }
     case PL_ACTION_WRITE:
@@ -3796,10 +3865,26 @@ PL_action(int action, ...)
     case PL_ACTION_ATTACH_CONSOLE:
     {
 #ifdef O_PLMT
-      return attachConsole();
+      rval = attachConsole();
 #else
-      return FALSE;
+      rval = FALSE;
 #endif
+      break;
+    }
+    case PL_GMP_SET_ALLOC_FUNCTIONS:
+    { int set = va_arg(args, int);
+
+#ifdef O_GMP
+      if ( !GD->gmp.initialised )
+      { GD->gmp.keep_alloc_functions = !set;
+	initGMP();
+      } else
+      { rval = FALSE;
+      }
+#else
+      rval = FALSE;
+#endif
+      break;
     }
     default:
       sysError("PL_action(): Illegal action: %d", action);
