@@ -85,7 +85,7 @@ COMMON(void) 		do_undo(mark *m);
 COMMON(Definition) 	getProcDefinition(Procedure proc);
 COMMON(void) 		fix_term_ref_count(void);
 COMMON(fid_t) 		PL_open_signal_foreign_frame(void);
-COMMON(bool)		foreignWakeup(ARG1_LD);
+COMMON(bool)		foreignWakeup(ARG1_LD, term_t *ex);
 
 /* pl-atom.c */
 COMMON(word) 		lookupAtom(const char *s, size_t len);
@@ -142,8 +142,9 @@ COMMON(Clause) 		compileClause(Word head, Word body,
 			      Procedure proc, Module module ARG_LD);
 COMMON(Clause) 		assert_term(term_t term, int where,
 				    SourceLoc loc ARG_LD);
-COMMON(void) 		unregisterAtomsClause(Clause clause);
+COMMON(void) 		forAtomsInClause(Clause clause, void (func)(atom_t a));
 COMMON(word) 		pl_redefine_system_predicate(term_t term);
+COMMON(Code)		stepDynPC(Code PC, const code_info *ci);
 COMMON(bool) 		decompileHead(Clause clause, term_t head);
 COMMON(int) 		arg1Key(Clause clause, word *key);
 COMMON(bool) 		decompile(Clause clause, term_t term, term_t bindings);
@@ -154,23 +155,13 @@ COMMON(word) 		pl_clause3(term_t p, term_t term, term_t ref,
 COMMON(word) 		pl_clause2(term_t p, term_t term, control_t h);
 COMMON(word) 		pl_nth_clause(term_t p, term_t n, term_t ref,
 				      control_t h);
-COMMON(word) 		pl_xr_member(term_t ref, term_t term, control_t h);
 COMMON(void) 		wamListClause(Clause clause);
 COMMON(Code) 		wamListInstruction(IOSTREAM *out, Clause clause,
 					   Code bp);
-COMMON(word) 		pl_wam_list(term_t ref);
-COMMON(word) 		pl_fetch_vm(term_t ref, term_t offset, term_t noffset,
-				    term_t instruction);
 COMMON(int) 		unify_definition(term_t head, Definition def,
 				 term_t thehead, int flags);
-COMMON(word) 		pl_clause_term_position(term_t ref, term_t pc,
-						term_t locterm);
-COMMON(word) 		pl_break_pc(term_t ref, term_t pc, term_t nextpc,
-				    control_t h);
-COMMON(word) 		pl_break_at(term_t ref, term_t pc, term_t set);
 COMMON(code) 		replacedBreak(Code PC);
 COMMON(void) 		clearBreakPointsClause(Clause clause);
-COMMON(word) 		pl_current_break(term_t ref, term_t pc, control_t h);
 COMMON(int) 		get_clause_ptr_ex(term_t ref, Clause *cl);
 COMMON(int)		unify_functor(term_t t, functor_t fd, int how);
 
@@ -249,6 +240,7 @@ COMMON(int) 		openFileDescriptors(unsigned char *buf, int size);
 COMMON(void) 		protocol(const char *s, size_t n);
 COMMON(bool) 		getInputStream__LD(term_t t, IOSTREAM **s ARG_LD);
 COMMON(bool) 		getOutputStream(term_t t, IOSTREAM **s);
+COMMON(bool)	        reportStreamError(IOSTREAM *s);
 COMMON(bool) 		streamStatus(IOSTREAM *s);
 COMMON(atom_t) 		fileNameStream(IOSTREAM *s);
 COMMON(int) 		getSingleChar(IOSTREAM *s, int signals);
@@ -408,6 +400,8 @@ COMMON(void)            bindExtensions(const char *module,
 				       const PL_extension *ext);
 COMMON(void)		initForeign(void);
 COMMON(int)		PL_rethrow(void);
+COMMON(int)		PL_pending__LD(int sig ARG_LD);
+COMMON(int)		PL_clearsig__LD(int sig ARG_LD);
 
 
 /* pl-fmt.c */
@@ -542,7 +536,7 @@ COMMON(int) 		Pause(real time);
 /* pl-prims.c */
 COMMON(bool) 		raw_unify_ptrs(Word t1, Word t2 ARG_LD);
 COMMON(bool) 		unify_ptrs(Word t1, Word t2 ARG_LD);
-COMMON(bool) 		can_unify(Word t1, Word t2);
+COMMON(bool) 		can_unify(Word t1, Word t2, term_t *ex);
 COMMON(int) 		compareStandard(Word t1, Word t2, int eq ARG_LD);
 COMMON(intptr_t)	skip_list(Word l, Word *tailp ARG_LD);
 COMMON(intptr_t) 	lengthList(term_t list, int errors);
@@ -710,6 +704,8 @@ COMMON(void) 		unblockSignal(int sig);
 COMMON(void) 		blockSignal(int sig);
 COMMON(void) 		resetSignals(void);
 COMMON(void) 		cleanupSignals(void);
+COMMON(int)		handleSignals(Code PC);
+
 COMMON(int) 		initPrologStacks(intptr_t local,
 				 intptr_t global,
 				 intptr_t trail,
@@ -723,10 +719,8 @@ COMMON(void) 		emptyStacks(void);
 COMMON(void) 		freeStacks(ARG1_LD);
 COMMON(void) 		freeLocalData(PL_local_data_t *ld);
 COMMON(word) 		pl_trim_stacks(void);
-COMMON(word) 		pl_limit_stack(term_t s, term_t l);
 COMMON(word) 		pl_stack_parameter(term_t s, term_t k, term_t o, term_t n);
 COMMON(void) 		ensureRoomStack(Stack s, size_t n);
-COMMON(int) 		_PL_get_signum(term_t sig, int *n);
 
 /* pl-sys.c */
 COMMON(word) 		pl_shell(term_t command, term_t status);
@@ -817,7 +811,6 @@ COMMON(word) 		pl_qlf_start_file(term_t name);
 COMMON(word) 		pl_qlf_end_part(void);
 COMMON(word) 		pl_qlf_open(term_t file);
 COMMON(word) 		pl_qlf_close(void);
-COMMON(word) 		pl_qlf_load(term_t file, term_t module);
 COMMON(word) 		pl_qlf_assert_clause(term_t ref, term_t saveclass);
 
 COMMON(void)		wicPutNum(int64_t n, IOSTREAM *fd);
@@ -937,3 +930,5 @@ COMMON(void)    promoteNumber(Number n1, numtype type);
 COMMON(int)	cmpNumbers(Number n1, Number n2);
 COMMON(void)	cpNumber(Number to, Number from);
 
+/* pl-version.h */
+COMMON(void)	setGITVersion(void);
