@@ -63,7 +63,7 @@ before loading this file.  See end of this file.
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 50654
+#define PLVERSION 50661
 #endif
 
 		 /*******************************
@@ -127,6 +127,7 @@ typedef Record		record_t;	/* handle to a recorded term */
 typedef struct PL_local_data *PL_engine_t; /* handle to a engine */
 #else
 typedef	uintptr_t	atom_t;		/* Prolog atom */
+typedef uintptr_t	functor_t;	/* Name/arity pair */
 typedef void *		module_t;	/* Prolog module */
 typedef void *		predicate_t;	/* Prolog procedure */
 typedef void *		record_t;	/* Prolog recorded term */
@@ -136,7 +137,6 @@ typedef uintptr_t	PL_fid_t;	/* opaque foreign context handle */
 typedef void *		control_t;	/* non-deterministic control arg */
 typedef void *		PL_engine_t;	/* opaque engine handle */
 #endif
-typedef uintptr_t	functor_t;	/* Name/arity pair */
 typedef uintptr_t	PL_atomic_t;	/* same a word */
 typedef uintptr_t	foreign_t;	/* return type of foreign functions */
 typedef wchar_t	        pl_wchar_t;	/* Prolog wide character */
@@ -378,6 +378,7 @@ PL_EXPORT(int)		PL_get_nchars(term_t t,
 				      unsigned int flags);
 PL_EXPORT(int)		PL_get_integer(term_t t, int *i);
 PL_EXPORT(int)		PL_get_long(term_t t, long *i);
+PL_EXPORT(int)		PL_get_intptr(term_t t, intptr_t *i);
 PL_EXPORT(int)		PL_get_pointer(term_t t, void **ptr);
 PL_EXPORT(int)		PL_get_float(term_t t, double *f);
 PL_EXPORT(int)		PL_get_functor(term_t t, functor_t *f);
@@ -747,6 +748,7 @@ install_t		PL_install_readline(void);
 #endif
 PL_EXPORT(int)		PL_toplevel(void);
 PL_EXPORT(int)		PL_cleanup(int status);
+PL_EXPORT(void)		PL_cleanup_fork();
 PL_EXPORT(void)		PL_halt(int status) NORETURN;
 
 		 /*******************************
@@ -796,14 +798,12 @@ PL_EXPORT(void)		PL_free(void *mem);
 typedef int  (*PL_dispatch_hook_t)(int fd);
 typedef void (*PL_abort_hook_t)(void);
 typedef void (*PL_initialise_hook_t)(int argc, char **argv);
-typedef void (*PL_async_hook_t)(void);	/* Win32 only (O_ASYNC_HOOK) */
 typedef int  (*PL_agc_hook_t)(atom_t a);
 
 PL_EXPORT(PL_dispatch_hook_t) 	PL_dispatch_hook(PL_dispatch_hook_t);
 PL_EXPORT(void)	       		PL_abort_hook(PL_abort_hook_t);
 PL_EXPORT(void)	       		PL_initialise_hook(PL_initialise_hook_t);
 PL_EXPORT(int)		      	PL_abort_unhook(PL_abort_hook_t);
-PL_EXPORT(PL_async_hook_t)    	PL_async_hook(unsigned int, PL_async_hook_t);
 PL_EXPORT(PL_agc_hook_t)      	PL_agc_hook(PL_agc_hook_t);
 
 
@@ -817,6 +817,8 @@ PL_EXPORT(void) (*PL_signal(int sig, void (*func)(int)))(int);
 PL_EXPORT(void) PL_interrupt(int sig);
 PL_EXPORT(int)  PL_raise(int sig);
 PL_EXPORT(int)  PL_handle_signals(void);
+PL_EXPORT(int) 	PL_get_signum_ex(term_t sig, int *n);
+
 
 		/********************************
 		*      PROLOG ACTION/QUERY      *
@@ -875,6 +877,8 @@ typedef struct
 
 
 PL_EXPORT(int)	PL_thread_self(void);	/* Prolog thread id (-1 if none) */
+PL_EXPORT(int)  PL_unify_thread_id(term_t t, int i);
+PL_EXPORT(int)	PL_get_thread_id_ex(term_t t, int *idp);
 PL_EXPORT(int)	PL_thread_attach_engine(PL_thread_attr_t *attr);
 PL_EXPORT(int)	PL_thread_destroy_engine(void);
 PL_EXPORT(int)	PL_thread_at_exit(void (*function)(void *),
