@@ -89,8 +89,9 @@ typedef struct message_queue
   pthread_cond_t       cond_var;	/* condvar for reading */
   pthread_cond_t       drain_var;	/* condvar for writing */
 #endif
-  struct _thread_msg   *head;		/* Head of message queue */
-  struct _thread_msg   *tail;		/* Tail of message queue */
+  struct thread_message   *head;		/* Head of message queue */
+  struct thread_message   *tail;		/* Tail of message queue */
+  uint64_t	       sequence_next;	/* next for sequence id */
   word		       id;		/* Id of the queue */
   long		       size;		/* # terms in queue */
   long		       max_size;	/* Max # terms in queue */
@@ -307,7 +308,6 @@ extern word		pl_thread_create(term_t goal, term_t id,
 					 term_t options);
 extern word		pl_thread_join(term_t thread, term_t retcode);
 extern word		pl_thread_exit(term_t retcode);
-extern word		pl_thread_kill(term_t thread, term_t sig);
 extern foreign_t	pl_thread_signal(term_t thread, term_t goal);
 
 extern foreign_t	pl_thread_at_exit(term_t goal);
@@ -439,5 +439,7 @@ extern void		initPrologThreads(void);
 #define PL_cons_list(l, h, t)	PL_cons_list__LD(l, h, t PASS_LD)
 #define PL_get_text(l, t, f)	PL_get_text__LD(l, t, f PASS_LD)
 #define PL_unify_int64_ex(t, i)	PL_unify_int64_ex__LD(t, i PASS_LD)
+#define PL_pending(sig)	        PL_pending__LD(sig PASS_LD)
+#define PL_clearsig(sig)        PL_clearsig__LD(sig PASS_LD)
 
 #endif /*PL_THREAD_H_DEFINED*/
