@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2006, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -1015,96 +1015,100 @@ loadPredicate(IOSTREAM *fd, int skip ARG_LD)
 
 	while( bp < ep )
 	{ code op = getInt(fd);
+	  const char *ats;
 	  int n = 0;
-	  int narg = codeTable[op].arguments;
 	  
-	  DEBUG(3, Sdprintf("\t%s (%d args) from %ld\n",
-			    codeTable[op].name, narg, Stell(fd)));
+	  if ( op >= I_HIGHEST )
+	    fatalError("Illegal op-code (%d) at %ld", op, Stell(fd));
+
+	  ats = codeTable[op].argtype;
+	  DEBUG(3, Sdprintf("\t%s from %ld\n", codeTable[op].name, Stell(fd)));
 	  *bp++ = encode(op);
-	  switch(codeTable[op].argtype)
-	  { case CA1_PROC:
-	    { *bp++ = loadXR(fd);
-	      n++;
-	      break;
-	    }
-	    case CA1_FUNC:
-	    case CA1_DATA:
-	    { word w = loadXR(fd);
-	      if ( isAtom(w) )
-		PL_register_atom(w);
-	      *bp++ = w;
-	      n++;
-	      break;
-	    }
-	    case CA1_MODULE:
-	      *bp++ = loadXR(fd);
-	      n++;
-	      break;
-	    case CA1_INTEGER:
-	      *bp++ = getLong(fd);
-	      n++;
-	      break;
-	    case CA1_INT64:
-	    { int64_t val = getInt64(fd);
-	      Word p = (Word)&val;
-	      
-	      cpInt64Data(bp, p);
-	      n += WORDS_PER_INT64;
-	      break;
-	    }
-	    case CA1_FLOAT:
-	    { union
-	      { word w[WORDS_PER_DOUBLE];
-		double f;
-	      } v;
-	      Word p = v.w;
-	      v.f = getReal(fd);
-	      cpDoubleData(bp, p);
-	      n += WORDS_PER_DOUBLE;
-	      break;
-	    }
-	    case CA1_STRING:		/* <n> chars */
-	    { int l = getInt(fd);
-	      int lw = (l+sizeof(word))/sizeof(word);
-	      int pad = (lw*sizeof(word) - l);
-	      char *s = (char *)&bp[1];
+	  DEBUG(0, assert(codeTable[op].arguments == VM_DYNARGC ||
+			  (size_t)codeTable[op].arguments == strlen(ats)));
 
-	      DEBUG(3, Sdprintf("String of %ld bytes\n", l));
-	      *bp = mkStrHdr(lw, pad);
-	      bp += lw;
-	      *bp++ = 0L;
-	      while(--l >= 0)
-		*s++ = Getc(fd);
-	      n++;
-	      break;
-	    }
-	    case CA1_MPZ:
+	  for(n=0; ats[n]; n++)
+	  { switch(ats[n])
+	    { case CA1_PROC:
+	      { *bp++ = loadXR(fd);
+		break;
+	      }
+	      case CA1_FUNC:
+	      case CA1_DATA:
+	      { word w = loadXR(fd);
+		if ( isAtom(w) )
+		  PL_register_atom(w);
+		*bp++ = w;
+		break;
+	      }
+	      case CA1_MODULE:
+		*bp++ = loadXR(fd);
+		break;
+	      case CA1_INTEGER:
+	      case CA1_JUMP:
+	      case CA1_VAR:
+	      case CA1_AFUNC:
+		*bp++ = (intptr_t)getInt64(fd);
+		break;
+	      case CA1_INT64:
+	      { int64_t val = getInt64(fd);
+		Word p = (Word)&val;
+		
+		cpInt64Data(bp, p);
+		break;
+	      }
+	      case CA1_FLOAT:
+	      { union
+		{ word w[WORDS_PER_DOUBLE];
+		  double f;
+		} v;
+		Word p = v.w;
+		v.f = getReal(fd);
+		cpDoubleData(bp, p);
+		break;
+	      }
+	      case CA1_STRING:		/* <n> chars */
+	      { int l = getInt(fd);
+		int lw = (l+sizeof(word))/sizeof(word);
+		int pad = (lw*sizeof(word) - l);
+		char *s = (char *)&bp[1];
+  
+		DEBUG(3, Sdprintf("String of %ld bytes\n", l));
+		*bp = mkStrHdr(lw, pad);
+		bp += lw;
+		*bp++ = 0L;
+		while(--l >= 0)
+		  *s++ = Getc(fd);
+		break;
+	      }
+	      case CA1_MPZ:
 #ifdef O_GMP
-	    DEBUG(3, Sdprintf("Loading MPZ from %ld\n", Stell(fd)));
-	    { int mpsize = getInt(fd);
-	      int l      = abs(mpsize)*sizeof(mp_limb_t);
-	      int wsz	 = (l+sizeof(word)-1)/sizeof(word);
-	      word m     = mkIndHdr(wsz+1, TAG_INTEGER);
-	      char *s;
-
-	      *bp++     = m;
-	      *bp++     = mpsize;
-	      s         = (char*)bp;
-	      bp[wsz-1] = 0L;
-	      bp       += wsz;
-
-	      while(--l >= 0)
-		*s++ = Getc(fd);
-	      DEBUG(3, Sdprintf("Loaded MPZ to %ld\n", Stell(fd)));
-	      n++;
-	      break;
-	    }
+	      DEBUG(3, Sdprintf("Loading MPZ from %ld\n", Stell(fd)));
+	      { int mpsize = getInt(fd);
+		int l      = abs(mpsize)*sizeof(mp_limb_t);
+		int wsz	 = (l+sizeof(word)-1)/sizeof(word);
+		word m     = mkIndHdr(wsz+1, TAG_INTEGER);
+		char *s;
+  
+		*bp++     = m;
+		*bp++     = mpsize;
+		s         = (char*)bp;
+		bp[wsz-1] = 0L;
+		bp       += wsz;
+  
+		while(--l >= 0)
+		  *s++ = Getc(fd);
+		DEBUG(3, Sdprintf("Loaded MPZ to %ld\n", Stell(fd)));
+		break;
+	      }
 #else
-  	      fatalError("No support for MPZ numbers");
+		fatalError("No support for MPZ numbers");
 #endif
+	      default:
+		fatalError("No support for VM argtype %d (arg %d of %s)",
+			   ats[n], n, codeTable[op].name);
+	    }
 	  }
-	  for( ; n < narg; n++ )
-	    *bp++ = getInt(fd);
 	}
 
 	if ( skip )
@@ -1761,96 +1765,92 @@ saveWicClause(Clause clause, IOSTREAM *fd)
 
   while( bp < ep )
   { code op = decode(*bp++);
-    int n = 0;
+    const char *ats = codeTable[op].argtype;
+    int n;
 
     putNum(op, fd);
-    DEBUG(3, Sdprintf("\t%s (%d args) at %ld\n", 
-		      codeTable[op].name, codeTable[op].arguments,
-		      Stell(fd)));
-    switch(codeTable[op].argtype)
-    { case CA1_PROC:
-      { Procedure p = (Procedure) *bp++;
-	n++;
-	saveXRProc(p, fd PASS_LD);
-	break;
-      }
-      case CA1_MODULE:
-      { Module m = (Module) *bp++;
-	n++;
-	saveXRModule(m, fd PASS_LD);
-	break;
-      }
-      case CA1_FUNC:
-      { functor_t f = (functor_t) *bp++;
-	n++;
-	saveXRFunctor(f, fd PASS_LD);
-	break;
-      }
-      case CA1_DATA:
-      { word xr = (word) *bp++;
-	n++;
-	saveXR(xr, fd);
-	break;
-      }
-      case CA1_INTEGER:
-      { putNum(*bp++, fd);
-	n++;
-	break;
-      }
-      case CA1_INT64:
-      { int64_t val;
-	Word p = (Word)&val;
-
-	cpInt64Data(p, bp);
-	n += WORDS_PER_INT64;
-	putNum(val, fd);
-	break;
-      }
-      case CA1_FLOAT:
-      { union
-	{ word w[WORDS_PER_DOUBLE];
-	  double f;
-	} v;
-	Word p = v.w;
-	cpDoubleData(p, bp);
-	n += WORDS_PER_DOUBLE;
-	putReal(v.f, fd);
-	break;
-      }
-      case CA1_STRING:
-      { word m = *bp;
-	char *s = (char *)++bp;
-	size_t wn = wsizeofInd(m);
-	size_t l = wn*sizeof(word) - padHdr(m);
-	bp += wn;
-
-	putNum(l, fd);
-	while(l-- > 0)
-	  Sputc(*s++&0xff, fd);
-	n++;
-	break;
-      }
+    DEBUG(3, Sdprintf("\t%s at %ld\n", codeTable[op].name, Stell(fd)));
+    for(n=0; ats[n]; n++)
+    { switch(ats[n])
+      { case CA1_PROC:
+	{ Procedure p = (Procedure) *bp++;
+	  saveXRProc(p, fd PASS_LD);
+	  break;
+	}
+	case CA1_MODULE:
+	{ Module m = (Module) *bp++;
+	  saveXRModule(m, fd PASS_LD);
+	  break;
+	}
+	case CA1_FUNC:
+	{ functor_t f = (functor_t) *bp++;
+	  saveXRFunctor(f, fd PASS_LD);
+	  break;
+	}
+	case CA1_DATA:
+	{ word xr = (word) *bp++;
+	  saveXR(xr, fd);
+	  break;
+	}
+	case CA1_INTEGER:
+	case CA1_JUMP:
+	case CA1_VAR:
+	case CA1_AFUNC:
+	{ putNum(*bp++, fd);
+	  break;
+	}
+	case CA1_INT64:
+	{ int64_t val;
+	  Word p = (Word)&val;
+  
+	  cpInt64Data(p, bp);
+	  putNum(val, fd);
+	  break;
+	}
+	case CA1_FLOAT:
+	{ union
+	  { word w[WORDS_PER_DOUBLE];
+	    double f;
+	  } v;
+	  Word p = v.w;
+	  cpDoubleData(p, bp);
+	  putReal(v.f, fd);
+	  break;
+	}
+	case CA1_STRING:
+	{ word m = *bp;
+	  char *s = (char *)++bp;
+	  size_t wn = wsizeofInd(m);
+	  size_t l = wn*sizeof(word) - padHdr(m);
+	  bp += wn;
+  
+	  putNum(l, fd);
+	  while(l-- > 0)
+	    Sputc(*s++&0xff, fd);
+	  break;
+	}
 #ifdef O_GMP
-      case CA1_MPZ:
-      { word m = *bp++;
-	size_t wn = wsizeofInd(m);
-	int mpsize = (int)*bp;
-	int l = abs(mpsize)*sizeof(mp_limb_t);
-	char *s = (char*)&bp[1];
-	bp += wn;
-
-	DEBUG(3, Sdprintf("Saving MPZ from %ld\n", Stell(fd)));
-	putNum(mpsize, fd);
-	while(--l >= 0)
-	  Sputc(*s++&0xff, fd);
-	DEBUG(3, Sdprintf("Saved MPZ to %ld\n", Stell(fd)));
-	n++;
-	break;
-      }
+	case CA1_MPZ:
+	{ word m = *bp++;
+	  size_t wn = wsizeofInd(m);
+	  int mpsize = (int)*bp;
+	  int l = abs(mpsize)*sizeof(mp_limb_t);
+	  char *s = (char*)&bp[1];
+	  bp += wn;
+  
+	  DEBUG(3, Sdprintf("Saving MPZ from %ld\n", Stell(fd)));
+	  putNum(mpsize, fd);
+	  while(--l >= 0)
+	    Sputc(*s++&0xff, fd);
+	  DEBUG(3, Sdprintf("Saved MPZ to %ld\n", Stell(fd)));
+	  break;
+	}
 #endif
+	default:
+	  fatalError("No support for VM argtype %d (arg %d of %s)",
+		     ats[n], n, codeTable[op].name);
+      }
     }
-    for( ; n < codeTable[op].arguments; n++ )
-      putNum(*bp++, fd);
   }
 }
 
@@ -1966,11 +1966,6 @@ addClauseWic(term_t term, atom_t file ARG_LD)
   if ( (clause = assert_term(term, CL_END, &loc PASS_LD)) )
   { IOSTREAM *s = wicFd;
 
-#ifdef O_DEBUGGER
-    DEBUG(3, Sdprintf("WAM code:\n");
-	     wamListClause(clause));
-#endif
-
     openProcedureWic(clause->procedure, s, ATOM_development PASS_LD);
     saveWicClause(clause, s);
 
@@ -2278,28 +2273,33 @@ popPathTranslation()
 }
 
 static bool
-qlfLoad(char *file, Module *module ARG_LD)
-{ IOSTREAM *fd;
-  bool rval;
+qlfLoad(IOSTREAM *fd, Module *module ARG_LD)
+{ bool rval;
   int lversion;
-  char *absloadname;
+  const char *absloadname;
   char tmp[MAXPATHLEN];
   int saved_wsize;
+  atom_t file;
 
-  wicFile = file;
-  if ( !(absloadname = AbsoluteFile(wicFile, tmp)) )
-    fail;
-  
-  if ( !(fd = Sopen_file(file, "rbr")) )
-  { term_t f = PL_new_term_ref();
+  if ( (file = fileNameStream(fd)) )
+  { PL_chars_t text;
 
-    PL_put_atom_chars(f, file);
-    return PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
-		    ATOM_open, ATOM_source_sink, f);
+    if ( !get_atom_text(file, &text) )
+      fail;
+    if ( !PL_mb_text(&text, REP_FN) )
+    { PL_free_text(&text);
+      fail;
+    }
+    wicFile = text.text.t;
+    if ( !(absloadname = AbsoluteFile(wicFile, tmp)) )
+      fail;
+    PL_free_text(&text);
+  } else
+  { absloadname = NULL;
   }
+  
   if ( !(lversion = qlfVersion(fd)) || lversion < LOADVERSION )
-  { Sclose(fd);
-    if ( lversion )
+  { if ( lversion )
       warning("$qlf_load/1: %s bad version (file version = %d, prolog = %d)",
 	      wicFile, lversion, VERSION);
     fail;
@@ -2321,8 +2321,6 @@ qlfLoad(char *file, Module *module ARG_LD)
   popXrIdTable(PASS_LD1);
   popPathTranslation();
 
-  Sclose(fd);
-
   return rval;
 }
 
@@ -2476,31 +2474,42 @@ pl_qlf_close()
 }
 
 
-word
-pl_qlf_load(term_t file, term_t module)
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+$qlf_load(:Stream, -ModuleOut)
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static
+PRED_IMPL("$qlf_load", 2, qlf_load, PL_FA_TRANSPARENT)
 { GET_LD
+  term_t qstream = A1;
+  term_t module = A2;
   Module m, oldsrc = LD->modules.source;
-  char *fn;
   bool rval;
-  term_t name = PL_new_term_ref();
+  term_t stream = PL_new_term_ref();
+  IOSTREAM *fd;
+  IOENC saved_enc;
 
   m = oldsrc;
-  if ( !PL_strip_module(file, &m, name) )
+  if ( !PL_strip_module(qstream, &m, stream) )
     fail;
-  if ( !PL_get_file_name(name, &fn, 0) )
+  if ( !PL_get_stream_handle(stream, &fd) )
     fail;
 
+  saved_enc = fd->encoding;
+  fd->encoding = ENC_OCTET;
   LD->modules.source = m;
-  rval = qlfLoad(fn, &m PASS_LD);
+  rval = qlfLoad(fd, &m PASS_LD);
   LD->modules.source = oldsrc;
+  fd->encoding = saved_enc;
 
-  if ( !rval )
-    fail;
+  if ( rval )
+  { if ( m )
+      return PL_unify_atom(module, m->name);
 
-  if ( m )
-    return PL_unify_atom(module, m->name);
-  else
     return PL_unify_integer(module, 0);
+  }
+
+  fail;
 }
 
 
@@ -2785,4 +2794,5 @@ wicPutStringW(const pl_wchar_t *w, size_t len, IOSTREAM *fd)
 
 BeginPredDefs(wic)
   PRED_DEF("$qlf_info", 5, qlf_info, 0)
+  PRED_DEF("$qlf_load", 2, qlf_load, PL_FA_TRANSPARENT)
 EndPredDefs
