@@ -1,11 +1,11 @@
-/*  $Id: pl-write.c,v 1.97 2007/02/20 11:21:42 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -106,6 +106,10 @@ varName(term_t t, char *name)
 #define AT_SOLO		4
 #define AT_SPECIAL	5
 
+/* Note: this only deals with ISO Latin-1 atoms; wide atoms are handled
+   by writeUCSAtom()
+*/
+
 static int
 atomType(atom_t a, IOSTREAM *fd)
 { Atom atom = atomValue(a);
@@ -151,6 +155,8 @@ atomType(atom_t a, IOSTREAM *fd)
 		 *	 PRIMITIVE WRITES	*
 		 *******************************/
 
+#define TRUE_WITH_SPACE 2		/* OK, and emitted leading space before token */
+
 static bool
 Putc(int c, IOSTREAM *s)
 { return Sputcode(c, s) == EOF ? FALSE : TRUE;
@@ -190,34 +196,58 @@ and given character require a space to ensure a token-break.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static bool
-PutOpenToken(int c, IOSTREAM *s)
+needSpace(int c, IOSTREAM *s)
 { if ( c == EOF )
   { s->lastc = EOF;
-    return TRUE;
+    return FALSE;
   } else if ( s->lastc != EOF &&
 	      ((isAlphaW(s->lastc) && isAlphaW(c)) ||
 	       (isSymbolW(s->lastc) && isSymbolW(c)) ||
-	       (s->lastc != '(' && !isBlank(s->lastc) && c == '(')) )
-  { return Putc(' ', s);
+	       (s->lastc != '(' && !isBlank(s->lastc) && c == '(') ||
+	       (c == '\'' && isDigit(s->lastc))) )
+  { return TRUE;
+  }
+
+  return FALSE;
+}
+
+
+static int
+PutOpenToken(int c, IOSTREAM *s)
+{ if ( needSpace(c, s) )
+  { TRY(Putc(' ', s));
+    return TRUE_WITH_SPACE;
   }
 
   return TRUE;
 }
 
 
-static bool
+static int
 PutToken(const char *s, IOSTREAM *stream)
 { if ( s[0] )
-    return PutOpenToken(s[0], stream) && PutString(s, stream);
+  { int rc;
+
+    TRY(rc=PutOpenToken(s[0], stream));
+    TRY(PutString(s, stream));
+
+    return rc;
+  }
 
   return TRUE;
 }
 
 
-static bool
+static int
 PutTokenN(const char *s, size_t len, IOSTREAM *stream)
 { if ( len > 0 )
-    return PutOpenToken(s[0], stream) && PutStringN(s, len, stream);
+  { int rc;
+
+    TRY(rc=PutOpenToken(s[0], stream));
+    TRY(PutStringN(s, len, stream));
+
+    return rc;
+  }
 
   return TRUE;
 }
@@ -231,9 +261,14 @@ openbrace to avoid interpretation as a term.   E.g. not (a,b) instead of
 not(a,b).  Reported by Stefan.Mueller@dfki.de.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static bool
+static int
 PutOpenBrace(IOSTREAM *s)
-{ return PutOpenToken('(', s) && Putc('(', s);
+{ int rc;
+
+  TRY(rc=PutOpenToken('(', s));
+  TRY(Putc('(', s));
+
+  return rc;
 }
 
 
@@ -392,7 +427,7 @@ writeBlob(atom_t a, write_options *options)
 }
 
 
-static bool
+static int				/* FALSE, TRUE or TRUE_WITH_SPACE */
 writeAtom(atom_t a, write_options *options)
 { Atom atom = atomValue(a);
 
@@ -411,10 +446,15 @@ writeAtom(atom_t a, write_options *options)
       case AT_QUOTE:
       case AT_FULLSTOP:
       default:
-	return writeQuoted(options->out,
-			   atom->name,
-			   atom->length,
-			   '\'', options);
+      { int rc;
+
+	TRY(rc=PutOpenToken('\'', options->out));
+	TRY(writeQuoted(options->out,
+			atom->name,
+			atom->length,
+			'\'', options));
+	return rc;
+      }
     }
   } else
     return PutTokenN(atom->name, atom->length, options->out);
@@ -806,6 +846,60 @@ writeTerm(term_t t, int prec, write_options *options)
 }
 
 
+static bool
+writeList2(term_t list, write_options *options, int cyclic)
+{ term_t head = PL_new_term_ref();
+  term_t l    = PL_copy_term_ref(list);
+  
+  TRY(Putc('[', options->out));
+  for(;;)
+  { PL_get_list(l, head, l);
+    TRY(writeTerm(head, 999, options));
+
+    if ( PL_get_nil(l) )
+      break;
+    if ( ++options->depth >= options->max_depth && options->max_depth )
+      return PutString("|...]", options->out);
+    if ( !PL_is_functor(l, FUNCTOR_dot2) )
+    { TRY(Putc('|', options->out));
+      TRY(writeTerm(l, 999, options));
+      break;
+    }
+
+					/* cycle detection */
+    { Word addr =  address_of(l);
+
+      if ( has_visited(options->visited, addr) )
+      { return PutString("|**]", options->out);
+      } else if ( cyclic )
+      { visited *v = alloca(sizeof(*v));
+	v->address = addr;
+	v->next = options->visited;
+	options->visited = v;
+      }
+    }
+
+    TRY(PutString(", ", options->out));
+  }
+
+  return Putc(']', options->out);
+}
+
+
+static bool
+writeList(term_t list, write_options *options)
+{ visited *v = options->visited;
+  Word tail;
+  int rc;
+
+  skip_list(valTermRef(list), &tail PASS_LD);
+  rc = writeList2(list, options, isList(*tail));
+  options->visited = v;
+
+  return rc;
+}
+
+
 static bool
 writeTerm2(term_t t, int prec, write_options *options)
 { atom_t functor;
@@ -936,34 +1030,14 @@ writeTerm2(term_t t, int prec, write_options *options)
 	}
       } else if ( arity == 2 )
       { if ( functor == ATOM_dot )	/* [...] */
-	{ term_t head = PL_new_term_ref();
-	  term_t l    = PL_copy_term_ref(t);
-  
-	  TRY(Putc('[', out));
-	  for(;;)
-	  { PL_get_list(l, head, l);
-  
-	    TRY(writeTerm(head, 999, options));
-	    if ( PL_get_nil(l) )
-	      break;
-	    if ( ++options->depth >= options->max_depth && options->max_depth )
-	      return PutString("|...]", options->out);
-	    if ( !PL_is_functor(l, FUNCTOR_dot2) )
-	    { TRY(Putc('|', out));
-	      TRY(writeTerm(l, 999, options));
-	      break;
-	    }
-	    TRY(PutString(", ", out));
-	  }
-	  return Putc(']', out);
-	}
+	  return writeList(t, options);
   
 					  /* <term> op <term> */
 	if ( currentOperator(options->module, functor, OP_INFIX,
 			     &op_type, &op_pri) )
 	{ term_t l = PL_new_term_ref();
 	  term_t r = PL_new_term_ref();
-  
+	  
 	  PL_get_arg(1, t, l);
 	  PL_get_arg(2, t, r);
   
@@ -974,12 +1048,15 @@ writeTerm2(term_t t, int prec, write_options *options)
 				? op_pri-1 : op_pri, 
 			options));
 	  if ( functor == ATOM_comma )
-	  { TRY(Putc(',', out));
+	  { TRY(PutString(", ", out));
 	  } else
-	  { TRY(writeAtom(functor, options));
+	  { switch(writeAtom(functor, options))
+	    { case FALSE:
+		fail;
+	      case TRUE_WITH_SPACE:
+		TRY(Putc(' ', out));
+	    } 
 	  }
-	  if ( functor == ATOM_comma )
-	    TRY(Putc(' ', out));
 	  TRY(writeTerm(r, 
 			op_type == OP_XFX || op_type == OP_YFX
 				? op_pri-1 : op_pri, 
@@ -1033,6 +1110,7 @@ static const opt_spec write_term_options[] =
   { ATOM_backquoted_string, OPT_BOOL },
   { ATOM_attributes,	    OPT_ATOM },
   { ATOM_priority,	    OPT_INT },
+  { ATOM_partial,	    OPT_BOOL },
   { NULL_ATOM,	     	    0 }
 };
 
@@ -1047,6 +1125,7 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   atom_t mname    = ATOM_user;
   atom_t attr     = ATOM_nil;
   int  priority   = 1200;
+  bool partial    = FALSE;
   IOSTREAM *s;
   write_options options;
 
@@ -1055,7 +1134,7 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   if ( !scan_options(opts, 0, ATOM_write_option, write_term_options,
 		     &quoted, &ignore_ops, &numbervars, &portray,
 		     &charescape, &options.max_depth, &mname,
-		     &bqstring, &attr, &priority) )
+		     &bqstring, &attr, &priority, &partial) )
     fail;
 
   if ( attr == ATOM_nil )
@@ -1091,7 +1170,8 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   if ( bqstring )   options.flags |= PL_WRT_BACKQUOTED_STRING;
 
   options.out = s;
-  PutOpenToken(EOF, s);			/* reset this */
+  if ( !partial )
+    PutOpenToken(EOF, s);		/* reset this */
   if ( (options.flags & PL_WRT_QUOTED) && !(s->flags&SIO_REPPL) )
   { s->flags |= SIO_REPPL;
     writeTerm(term, priority, &options);
@@ -1157,7 +1237,7 @@ pl_write2(term_t stream, term_t term)
 
 word
 pl_writeq2(term_t stream, term_t term)
-{ return do_write2(stream, term, PL_WRT_QUOTED);
+{ return do_write2(stream, term, PL_WRT_QUOTED|PL_WRT_NUMBERVARS);
 }
 
 word
@@ -1212,4 +1292,30 @@ pl_writeln(term_t term)
 }
 
 
+static
+PRED_IMPL("$put_token", 2, put_token, 0)
+{ char *s;
+  size_t len;
+  IOSTREAM *out;
+
+  if ( !PL_get_stream_handle(A1, &out) )
+    fail;
+  if ( !PL_get_nchars(A2, &len, &s, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) )
+    fail;
+
+  if ( PutTokenN(s, len, out) )
+    return PL_release_stream(out);
+
+  PL_release_stream(out);
+  fail;
+}
+
+
+		 /*******************************
+		 *      PUBLISH PREDICATES	*
+		 *******************************/
+
+BeginPredDefs(write)
+  PRED_DEF("$put_token", 2, put_token, 0)
+EndPredDefs
 
